<?php

namespace Drupal\mdp\Plugin\Field\FieldType;

use Drupal\Core\Field\Attribute\FieldType;
use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\TypedData\ComplexDataDefinitionInterface;
use Drupal\Core\TypedData\DataDefinition;
use Drupal\Core\TypedData\TypedDataInterface;
use Drupal\mdp\Services\MDPService;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'mdp' field type.
 */
#[FieldType(
  id: "field_mdp",
  label: new TranslatableMarkup('Multi Date Picker (MDP)'),
  description: [
    new TranslatableMarkup('This field allows pick between multiple dates.'),
    new TranslatableMarkup('Ideal to schedule events, for example between 2 and 5 dates.'),
    new TranslatableMarkup('If pick a date this will be validate how a unique date, if pick only two dates these will be take how a range of dates and will validate all dates between them including the picked dates and if pick more three dates these will be validate each one.'),
  ],
  category: "date_time",
  default_widget: "field_mdp_widget",
  default_formatter: "field_mdp_formatter",
  list_class: "\Drupal\mdp\Plugin\Field\MDPItemList",
  cardinality: -1,
)]
class MDPItem extends FieldItemBase {

  /**
   * Value for the 'mdp_type' setting: store only a dates.
   */
  const MDP_ONLY_DATE = 'onlyDates';

  /**
   * The MDP service.
   *
   * @var \Drupal\mdp\Services\MDPService
   */
  protected $mdpService;

  /**
   * Constructs a MDPItem object.
   *
   * @param \Drupal\Core\TypedData\ComplexDataDefinitionInterface $definition
   *   The data definition.
   * @param string $name
   *   (optional) The name of the created property, or NULL if it is the root
   *   of a typed data tree. Defaults to NULL.
   * @param \Drupal\Core\TypedData\TypedDataInterface $parent
   *   (optional) The parent object of the data property, or NULL if it is the
   *   root of a typed data tree. Defaults to NULL.
   * @param \Drupal\mdp\Services\MDPService $mdpService
   *   The MDP service for date range validation.
   */
  public function __construct(ComplexDataDefinitionInterface $definition, $name = NULL, ?TypedDataInterface $parent = NULL, ?MDPService $mdpService = NULL) {
    parent::__construct($definition, $name, $parent);
    $this->mdpService = $mdpService;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration['definition'],
      $plugin_id,
      $plugin_definition,
      $container->get('mdp')
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {
    return [
      'columns' => [
        'value' => [
          'type' => 'text',
          'size' => 'big',
          'not null' => FALSE,
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultStorageSettings() {
    return [
      'mdp_type' => self::MDP_ONLY_DATE,
    ] + parent::defaultStorageSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function storageSettingsForm(array &$form, FormStateInterface $form_state, $has_data) {
    $element = [];
    $element['mdp_type'] = [
      '#type' => 'select',
      '#title' => $this->t('MDP type'),
      '#default_value' => $this->getSetting('mdp_type'),
      '#options' => [
        self::MDP_ONLY_DATE => $this->t('Only Dates'),
      ],
      '#disabled' => $has_data,
      '#description' => $this->t("The hours will be from 00:00 to 23:59 for each day."),
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultFieldSettings() {
    return [
      'separator' => ', ',
    ] + parent::defaultStorageSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function fieldSettingsForm(array $form, FormStateInterface $form_state) {
    $element = parent::fieldSettingsForm($form, $form_state);

    // $element['separator'] = [
    //   '#type' => 'select',
    //   '#title' => $this->t('Separator'),
    //   '#options' => [
    //     ', ' => $this->t('Comma'),
    //     '; ' => $this->t('Semicolon'),
    //     ' ' => $this->t('Space'),
    //     ' - ' => $this->t('Dash'),
    //     chr(8201) => $this->t('Thin space'),
    //   ],
    //   '#disabled' => true,
    //   '#default_value' => $this->getSetting('separator'),
    // ];
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    $properties['value'] = DataDefinition::create('string')
      ->setLabel('Dates')
      ->setDescription('Dates values.');

    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty() {
    $value = $this->getValue();
    return $value === NULL || $value === '';
  }

  /**
   * {@inheritdoc}
   */
  public function preSave() {
    if (strlen($this->value) > 0) {
      $separator = $this->getSetting('separator');
      $dates = explode($separator, $this->value);

      foreach ($dates as $key => $datetime) {
        $dates[$key] = strtotime($datetime);
      }

      $this->value = implode($separator, $dates);
    }
  }
}
