<?php

namespace Drupal\mdp\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\Attribute\FieldWidget;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\mdp\Services\MDPService;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'mdp_field' widget.
 */
#[FieldWidget(
  id: 'field_mdp_widget',
  label: new TranslatableMarkup('MDP - Widget'),
  field_types: ['field_mdp']
)]
class MDPDefaultWidget extends WidgetBase {

  /**
   * The MDP service.
   *
   * @var \Drupal\mdp\Services\MDPService
   */
  protected MDPService $mdpService;

  /**
   * MDPDefaultWidget constructor.
   *
   * @param string $plugin_id
   *   The plugin_id for the widget.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the widget is associated.
   * @param array $settings
   *   The widget settings.
   * @param array $third_party_settings
   *   Any third party settings.
   * @param \Drupal\mdp\Services\MDPService $mdpService
   *   The MDP service for date range validation.
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, array $third_party_settings, MDPService $mdpService) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings);
    $this->mdpService = $mdpService;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['third_party_settings'],
      $container->get('mdp')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  protected function formMultipleElements(FieldItemListInterface $items, array &$form, FormStateInterface $form_state) {
    $separator = $this->fieldDefinition->getSetting('separator');
    $dataSettings = [
      'separator' => $separator,
    ];

    $value = [];
    foreach ($items as $item) {
      array_push($value, $this->mdpService->convertTimestampToDate($item->value, $separator, TRUE));
    }

    $value = implode($separator, $value);

    $detail['mdp'] = [
      '#type' => 'details',
      '#title' => t('Multi Dates Picker'),
      '#open' => !empty($value) && strlen($value) > 0 ? TRUE : FALSE,
      '#group' => 'advanced',
      '#tree' => TRUE,
      '#weight' => -3,
    ];

    $detail['mdp']['value'] = [
      '#type' => 'textfield',
      '#default_value' => $value,
      '#attributes' => [
        'class' => ["edit-field-mdp-widget"],
        'autocomplete' => "off",
        'data-settings' => json_encode($dataSettings),
      ],
      '#maxlength' => 255,
      '#attached' => [
        'library' => [
          'mdp/mdp',
        ],
      ],
    ];

    $element = $detail;

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function extractFormValues(FieldItemListInterface $items, array $form, FormStateInterface $form_state) {
    $values = $form_state->getValue($this->fieldDefinition->getName());
    $values = $values['mdp'];
    $items->setValue($values['value']);
  }
}
