<?php

namespace Drupal\mdp\Plugin\views\filter;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\mdp\Services\MDPService;
use Drupal\views\Attribute\ViewsFilter;
use Drupal\views\Plugin\views\filter\FilterPluginBase;
use Drupal\views\Views;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Filter by mdp field
 *
 * @ingroup views_filter_handlers
 *
 */
#[ViewsFilter("mdp_filter")]
class MDPFilter extends FilterPluginBase implements ContainerFactoryPluginInterface{

  /**
   * The MDP service.
   *
   * @var \Drupal\mdp\Services\MDPService
   */
  protected $mdpService;

  /**
   * Constructs a new StringFilter object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\mdp\Services\MDPService $mdp_service
   *  The MDP service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, MDPService $mdp_service) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->mdpService = $mdp_service;
  }

  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('mdp')
    );
  }

  /**
   * Options form subform for setting options.
   *
   * This should be overridden by all child classes and it must
   * define $form['value']
   *
   * @param array $form
   *   An alterable, associative array containing the structure of the form,
   *   passed by reference.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @see buildOptionsForm()
   */
  protected function valueForm(&$form, FormStateInterface $form_state) {
    $form['value'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Value'),
      '#attributes' => [
        'class' => ['edit-field-mdp-widget'],
        'autocomplete' => 'off',
        'data-settings' => json_encode([
          'separator' => ', ',
        ]),
      ],
      '#default_value' => $this->value,
      '#attached' => [
        'library' => [
          'mdp/mdp'
        ],
      ],
    ];
  }

  /**
   * Add this filter to the query.
   *
   * Due to the nature of fapi, the value and the operator have an unintended
   * level of indirection. You will find them in $this->operator
   * and $this->value respectively.
   */
  public function query() {
    $this->ensureMyTable();
    $field = "node__field_multi_date_picker.$this->realField";

    /** @var \Drupal\views\Plugin\views\query\Sql $query */
    $query = $this->query;
    if (!empty($this->value[0])) {
      $dates = explode(',', $this->value[0]);

      foreach ($dates as $key => $datetime) {
        $dates[$key] = strtotime($datetime);
      }

      $join = Views::pluginManager('join')->createInstance('standard', [
        'table' => 'node__field_multi_date_picker',
        'field' => 'entity_id',
        'left_table' => 'node_field_data',
        'left_field' => 'nid',
        'operator' => '=',
      ]);
      $query->addRelationship('node__field_multi_date_picker', $join, 'node_field_data');
      $query->distinct = TRUE;

      switch (count($dates)) {
        case 1:
          $query->addWhere($this->options['group'], $field, $dates[0], '=');
          break;
        case 2;
          $query->addWhere($this->options['group'], $field, $dates[0], '>=');
          $query->addWhere($this->options['group'], $field, $dates[1], '<=');
          break;
        default:
          $query->addWhere($this->options['group'], $field, $dates, 'IN');
          break;
      }
    }
  }

}
